\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\-----------------------------------------------------------------------------

\\ Kevin Ryde, November 2021
\\
\\ Usage: gp <a349009.gp
\\
\\ Calculate terms of A349009, being decimal digits of the area "HAf" of the
\\ convex hull around the R5 dragon fractal.
\\
\\ The method is the formulas presented in the sequence with suitable scale
\\ factor 10^len.  The formulas are lightly adapted by moving "scale * 2/25"
\\ inside HAgrowf, since HAgrowf(z) is linear in the magnitude of z.
\\ 
\\ The loop end condition is the sum upper bound given in the sequence.
\\ The loop stops when "omitted" is too small to affect the requested "len"
\\ many digits.  frac(HAf) + omitted < 1 means the integer part omitted
\\ doesn't push HAf up to a different integer part which is the scaled
\\ output digits.

default(strictargs,1);
default(recover,0);

b = 1+2*I;

\\ return z rotated if necessary to put it in the 1st or 3rd quadrant
RotQ(z) = if(sign(real(z))==sign(imag(z)), z, z*I);

\\ return complex z with the imaginary part sheared by doubling
ShearIm(z) = z + I*imag(z);

MinReIm(z) = min(abs(real(z)), abs(imag(z)));

HAgrowf(z) = MinReIm(ShearIm(RotQ(z)));

\\ Return a vector of the first len many decimal digits of A349009 = HAf
a_vector(len) =
{
  my(scale = 10^len,       \\ to get len many decimal digits
     j = 0,
     HAf = scale * 17/25,  \\ start with constant part
     B = scale * 2/25,     \\ scale * 2/25 / b^j
     omitted = scale);     \\ scale * 1/sqrt(5)^j
  while(1,
        j++;
        if(j%2==0,
           omitted /= 5;
           \\ omitted == scale/5^(j/2) || error();
           if(frac(HAf) + omitted < 1,
              return(digits(floor(HAf)))));
        B /= b;
        HAf += 3*HAgrowf(      B)
             +   HAgrowf((4+I)*B)) ;
}

{
  my(v=a_vector(24));
  print("A349009 digits");
  print1("  ");
  for(i=1,#v, print1(v[i],", "));
  print("...");
  print("  = 0.",fromdigits(v),"...");
}

print("end");
