#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'sum';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'A','TurnRuns3','xy_rotate_plus90','rect_corners_sort';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 3;
if (@ARGV) { $level = shift @ARGV; }

my %type_to_colour = ('0111' => undef,   # TurnRuns3
                      '0000' => undef,   # '[my grey]',    # 4 sep
                      '0011' => undef);  # 3 and 1
my $flavour = '31';
if (@ARGV) {
  $flavour = shift @ARGV;
}
if ($flavour eq '4') {
  $type_to_colour{'0000'} = '';
}
if ($flavour eq '31') {
  $type_to_colour{'0011L'} = '[my left]';
  $type_to_colour{'0011R'} = '[my right]';
}

print "  % generated by tools/$FindBin::Script $level $flavour\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $n_start = $n_lo;
my $n_end   = $n_hi;

if (@ARGV && $ARGV[0] eq 'quad') {
  $n_start = 4*$n_hi;
  $n_end   = 8*$n_hi;
}

### $n_start
### $n_end

{
  my ($x,$y) = $path->n_to_xy($n_start);
  print "  \\node (start) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_end);
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

# all of curve, some to be removed below
my $segments = MyLineSegments->new;
foreach my $n ($n_start .. $n_end-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

sub segments_remove_square {
  my ($segments, $x,$y) = @_;
  $segments->remove($x,$y,   $x+1,$y);
  $segments->remove($x,$y+1, $x+1,$y+1);
  $segments->remove($x,  $y, $x,  $y+1);
  $segments->remove($x+1,$y, $x+1,$y+1);
}

my %squares_lists;
my %seen;
foreach my $n ($n_start .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($sx,$sy) = xy_rotate_plus90($dx,$dy);
  foreach my $side ('L','R') {
    my $xo = $x + $dx + $sx;
    my $yo = $y + $dy + $sy;
    my ($x1,$y1, $x2,$y2) = rect_corners_sort ($x,$y, $xo,$yo);
    ### corner: "n=$n $side  $x1,$y1 to $x2,$y2  (from $x,$y $dx,$dy $sx,$sy)"
    if (! $seen{"$x1,$y1,$side"}++) {
      my $type = MyR5Dragon::square_consecutive_type_xy($x1,$y1,
                                                        $n_start,$n_end);
      ### $type
      if ($type ne '') {
        push @{$squares_lists{$type}}, [$x1,$y1];
      }
    }
    ($sx,$sy) = (-$sx,-$sy);
    ### sx: "$sx,$sy"
  }
}

# count = ('0111' => 0,
#              '0000' => 0,
#             );
foreach my $type (sort keys %squares_lists) {
  exists $type_to_colour{$type} // die "no colour for type $type";
  my $colour = $type_to_colour{$type} // next;

  my $aref = $squares_lists{$type} // next;
  next unless @$aref;
  $Printwrap = 2;
  printwrap("\\fill$colour foreach \\p in {");
  $Printwrap += 2;
  foreach my $p (@$aref) {
    my ($x,$y) = @$p;
    printwrap("($x,$y),");
    segments_remove_square($segments, $x,$y);
  }
  printwrap("}");
  printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

$Printwrap = 2;
printwrap("\\draw[my line] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";");
my $count = $segments->lines_joined_count_points;
printwrap(" % $count points\n");


my %counts = map {$_ => scalar @{$squares_lists{$_}}} keys %squares_lists;
my $A = sum(0, values %counts);
my $Sep4 = $counts{'0000'} || 0;
my $TurnRuns3 = $counts{'0111'} || 0;
my $A31 = ($counts{'0011L'} || 0) + ($counts{'0011R'} || 0);

if ($n_start == $n_lo) {
  $A == A($level) or die "oops, A";
  $Sep4 == A($level-1) or die "oops, Sep4";
  $TurnRuns3 == TurnRuns3($level) or die "oops, TurnRuns3";

  print "  \\newcommand{\\MyA}{$A}\n";
  print "  \\newcommand{\\MyTurnThree}{$TurnRuns3}\n";
  print "% GP-Test  A($level)==$A\n";
  print "% GP-Test  TurnRuns3($level)==$TurnRuns3\n";
  print "% GP-Test  A31($level)==$A31\n";
}

print "  % end generated\n";
exit 0;
