#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'xy_to_node_name', '@dir4_to_dx','@dir4_to_dy',
  'Hwpf_k2_hull_points', '$hullstouch_segs';
use MyPlanar;
use MyPrintwrap ':all';
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";

sub translate_hull {
  my ($x,$y, $dir) = @_;
  my $points = MyPlanar::points_rotate_plus90(Hwpf_k2_hull_points(), $dir);
  $points = MyPlanar::points_move($points, $x,$y);
}
scalar(@{translate_hull(0,0,0)}) == 10 || die "oops";


our $conf = MyR5DragonConf->new
  (name => 'hullsnontouch',
   segs => [
            '-1,0,1',   # top left
            '-1,1,0',
            '0,1,1',
            '1,2,3',    # top right
            '1,1,0',
            '2,1,3',

            '-1,0,3',    # bottom left
            '-1,-1,0',
            '0,-1,3',
            '1,-2,1',    # bottom right
            '1,-1,0',
            '2,-1,1',
           ]);
{
  my $str = $conf->print_to_str;
  $str =~ s/^/  %    /mg;
  $str =~ s/ +\n/\n/g;
  print $str;
  print "  %\n";
}
{
  my $union = $conf->union($hullstouch_segs)
    ->set_name('hullstouch and hullsnontouch union');
  my $str = $union->print_to_str;
  $str =~ s/^/  %    /mg;
  $str =~ s/ +\n/\n/g;
  print $str;
  print "  %\n";
}
$conf->intersection($hullstouch_segs)->num_segs == 0
  or die "oops";

{
  my $rectangle = [ [-2,  -3],
                    [ 3,  -3],
                    [ 3,   3],
                    [-2,   3]];
  my $inside_gpc = MyPlanar::points_to_gpc($rectangle);
  foreach my $x (-5 .. 6) {
    foreach my $y (-5 .. 5) {
      foreach my $dir (0 .. 3) {
        next unless ($x+$y+$dir)%2 == 0;
        my $seg = "$x,$y,$dir";
        MyR5DragonConf->seg_validate($seg);
        next if $hullstouch_segs->contains($seg) && $seg ne '0,0,0';
        my $seg_points = translate_hull($x,$y,$dir);
        my $seg_gpc = MyPlanar::points_to_gpc($seg_points);
        $inside_gpc = Math::Geometry::Planar::GpcClip('DIFFERENCE',
                                                      $inside_gpc, $seg_gpc);
      }
    }
  }

  my @inside_points = MyPlanar::gpc_to_points_list($inside_gpc);
  ### @inside_points

  my @points_list;
  my @contours = Math::Geometry::Planar::Gpc2Polygons($inside_gpc);
  @contours == 1 or die "oops, multiple contours";
  my $total_holes = 0;
  foreach my $contour (@contours) {  # Math::Geometry::Planar contour
    ### $contour
    # Second and subsequent polygon of the contour are holes.
    # Think their points already cycle the opposite way as required by pgf
    # default \fill[nonzero rule], but using \fill[even odd rule] works either
    # way.
    $Printwrap = 2;
    printwrap(" \\fill[my grey,even odd rule]");
    $Printwrap += 2;
    my $polygons = $contour->polygons;  # arrayref of points arrayrefs
    $total_holes += scalar(@$polygons)-1;
    my $separator = '';
    foreach my $points (@$polygons) {
      ### $points
      printwrap($separator);
      $separator = "\n";
      printwrap_poly($points);
      push @points_list, @$points;
    }
    printwrap(";\n");
  }

  my $inside_polygons = $contours[0]->polygons;
  ### inside_polygons: scalar(@$inside_polygons)
  ### $inside_polygons

  my $inside_hole_points = $inside_polygons->[1];
  # scalar(@$inside_hole_points) == 10 or die "oops";

  # $conf is all hulls which have some boundary touch of the $inside_gpc
  # formed (but are not in $hullstouch_segs);
  #
  my $inside_points = $inside_polygons->[0];
  foreach my $x (-3 .. 4) {
    foreach my $y (-3 .. 3) {
      foreach my $dir (0 .. 3) {
        next unless ($x+$y+$dir)%2 == 0;
        my $seg = "$x,$y,$dir";
        MyR5DragonConf->seg_validate($seg);
        next if $hullstouch_segs->contains($seg);
        my $seg_points = translate_hull($x,$y,$dir);

        my $by_segs    = $conf->contains($seg) ? 1 : 0;
        my $by_polygon = MyPlanar::points_has_boundary_touch($inside_points,
                                                             $seg_points)
          ? 1 : 0;
        unless ($by_polygon == $by_segs) {
          print "oops, by polygon = $by_polygon\n";
          print "      by segs    = $by_segs\n";
          die "different touch of inside_points";
        }
      }
    }
  }

  my $m_points = Hwpf_k2_hull_points();
  print "% GP-DEFINE  hullsnontouch_min_distance = ",
    MyPlanar::poly_distance_boundary_to_boundary($inside_points,$m_points),
      ";\n";
}


my %seen;
foreach my $seg ('0,0,0', sort $conf->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  foreach (0, 1) {
    my $name = xy_to_node_name($x,$y);
    unless ($seen{$name}++) {
      my $blank = ($x==0 && $y==0) || ($x==1 && $y==0) ? '' : ' blank';
      print "  \\node ($name) at ($x,$y) [my$blank dot];\n";
    }
    $x += $dir4_to_dx[$dir];
    $y += $dir4_to_dy[$dir];
  }
}

foreach my $seg ('0,0,0', sort $conf->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir4_to_dx[$dir], $y+$dir4_to_dy[$dir]);
  my $angle = 90*$dir;
  my $style = ($seg eq '0,0,0' ? 'my m arrow' : 'my arrow');
  print "  \\draw[$style] ($from)--($to);\n";
  print "  \\pic at ($from) [transform shape,rotate=$angle] {my Hwpf k2 hull pic};\n";
}

print "  % end generated\n";
exit 0;
